package com.jwipc.sys.convert;

import com.jwipc.sys.dto.DictDTO;
import com.jwipc.sys.dto.*;
import com.jwipc.sys.entity.Dict;
import com.jwipc.sys.entity.DictType;
import com.jwipc.sys.model.po.DictByTypePO;
import org.mapstruct.Mapper;

import java.util.List;


@Mapper(componentModel = "spring")
public interface DictConvert {
    List<DictDTO> byTypePOs2DTOs(List<DictByTypePO> dict);

    List<DictTypeListDTO> entities2ListDTOs(List<DictType> list);

    DictEditDTO entity2EditDTO(DictType byId);

    DictType editDTO2Entity(DictEditDTO dictEditDTO);

    List<DictItemListDTO> entities2ItemListDTOs(List<Dict> list);

    Dict itemEditDTO2eintity(DictItemEditDTO dictEditDTO);

    DictItemEditDTO entity2EditItemDTO(Dict byId);

    List<Dict> importDTOs2Entites(List<DictItemImportDTO> cachedDataList);
}
